#---------------------------------------------------------------
# Printer
#----------------------------------------------------------------

#
# Configurations

set pr(printerList) {
    { gv  { ghostview -geometry +0+0 } }
    { lp  { lpr                      } }
}
set pr(paperList) { A4 Letter }
set pr(positionList) { Center UpperLeft UpperRight LowerLeft LowerRight }
set pr(zoomRange) {10 300 1}
set pr(xoffsetRange) {0 300 1}
set pr(yoffsetRange) {0 300 1}
set pr(printer) gv
set pr(paper) A4
set pr(position) Center
set pr(zoom) 100
set pr(xoffset) 72
set pr(yoffset) 72
set pr(autoprint) 1
set pr(c) 0
set pr(viewer) $file(viewer)

foreach i $pr(printerList) { set pr([lindex $i 0]command) [lindex $i 1] }

#
# window_Printer and make_Printer

proc window_Printer { wn } { global pr c w l
    label $wn.ti -text "TkScan Print Manager" -width 24 -pady 4 -relief ridge
    canvas $wn.sep -width 0 -height 5 -relief flat
    pack  $wn.ti $wn.sep -side top

    frame $wn.pr -bd 1 -relief groove
    label $wn.pr.l -text Printer -width 9 -anchor w
    menubutton $wn.pr.mb -menu $wn.pr.mb.m \
                         -anchor w -width 8 -fg $l(c1) -relief ridge
    set w(printer) $wn.pr.mb
    pack $wn.pr.l $wn.pr.mb -side left
    menu $wn.pr.mb.m -tearoff false
    foreach i $pr(printerList) { set printer [lindex $i 0]
        $wn.pr.mb.m add command -label $printer -command "setPrinter $printer"
    }

    frame $wn.po -bd 1 -relief groove
    frame $wn.po.zo
    label $wn.po.zo.l -text "Zoom" -width 9 -anchor w
    entryscale $wn.po.zo pr(zoom) {} $pr(zoomRange) 113
    pack $wn.po.zo.l $wn.po.zo.e $wn.po.zo.s -side left
    frame $wn.po.po
    label $wn.po.po.l -text Position -width 9 -anchor w
    menubutton $wn.po.po.mb -menu $wn.po.po.mb.m \
                            -anchor w -width 10 -fg $l(c1) -relief ridge
    set w(printposition) $wn.po.po.mb
    pack  $wn.po.po.l $wn.po.po.mb -side left
    menu $wn.po.po.mb.m -tearoff false
    foreach i $pr(positionList) { set position [lindex $i 0]
        $wn.po.po.mb.m add command -label $position \
                                   -command "setPrintposition $position"
    }
    frame $wn.po.xo
    label $wn.po.xo.l -text "X offset" -width 9 -anchor e
    entryscale $wn.po.xo pr(xoffset) {} $pr(xoffsetRange) 113
    pack $wn.po.xo.l $wn.po.xo.e $wn.po.xo.s -side left
    frame $wn.po.yo
    label $wn.po.yo.l -text "Y offset" -width 9 -anchor e
    entryscale $wn.po.yo pr(yoffset) {} $pr(yoffsetRange) 113
    pack $wn.po.yo.l $wn.po.yo.e $wn.po.yo.s -side left
    pack $wn.po.zo $wn.po.po $wn.po.xo $wn.po.yo -side top -fill x
    set w(xoffsetlabel) $wn.po.xo.l
    set w(yoffsetlabel) $wn.po.yo.l

    frame $wn.pl -bd 1 -relief groove
    frame $wn.pl.b
    pack $wn.pl.b -side top -fill x
    set w(printpagelist) $wn.pl.b.b
    listbox $w(printpagelist) -width 19 -height 6 -font $l(f1) -fg blue \
            -yscrollcommand "$wn.pl.b.s set"
    pack $wn.pl.b.b -side left
    scrollbar $wn.pl.b.s -orient vertical -command "$w(printpagelist) yview"
    pack $wn.pl.b.s -side left -fill y
    frame $wn.pl.b.f
    pack $wn.pl.b.f -side right
    bind $w(printpagelist) <Double-Button-1> {
        if {[$w(printpagelist) curselection] != {}} {
            set i $pr([$w(printpagelist) curselection])
            do_print $i
        }
    }
    button $wn.pl.b.f.1 -text Remove -width 5 -pady 2 -command {
        if {[set t [$w(printpagelist) curselection]] != {}} {
            exec sh -c "rm -f $c(tkscandir)/[lindex $pr($t) 0].pnm"
            for {set i $t} {$i < [expr $pr(c)-1]} {incr i} {
                set pr($i) $pr([expr $i+1])
            }
            incr pr(c) -1
            update_printpagelist
            if {$t < $pr(c)} { $w(printpagelist) selection set $t
                      } else { $w(printpagelist) selection set [expr $t-1] }
        }
    }
    button $wn.pl.b.f.2 -text View   -width 5 -pady 2 -command {
        if {[set i [$w(printpagelist) curselection]] != {}} {
            set img [lindex $pr($i) 0].pnm
            exec sh -c "$pr(viewer) $c(tkscandir)/$img" &
        }
    }
    pack $wn.pl.b.f.1 $wn.pl.b.f.2 -side top

    frame $wn.b
    checkbutton $wn.b.0 -text Auto -variable pr(autoprint)
    button $wn.b.1 -text Print -width 21 -command {
        if {[set i [$w(printpagelist) curselection]] != {}} {do_print $i}
    }
    pack $wn.b.0 $wn.b.1 -side left -fill x

    pack $wn.pr $wn.po $wn.pl $wn.b -side top -fill x -expand yes

    setPrinter $pr(printer)
    setPrintposition $pr(position)
}

proc make_Printer { prefix i } { global pr s
    set tag "$prefix-$i"
    stat_text "Scanning $s(curscan) / $s(nrect) ... $tag.pnm"
    set pr($pr(c)) "$tag $s(tx$i) $s(ty$i) $s(bx$i) $s(by$i)"
    if $pr(autoprint) { do_print $pr(c) }
    incr pr(c)
    update_printpagelist
    update
}

#
# Subroutines

proc update_printpagelist {} { global pr w
    $w(printpagelist) delete 0 end
    for {set i 0} {$i < $pr(c)} {incr i} {
        $w(printpagelist) insert end [lindex $pr($i) 0].pnm
    }
}

proc do_print { i } { global pr s c
    set tag [lindex $pr($i) 0]
    set tx  [lindex $pr($i) 1]
    set ty  [lindex $pr($i) 2]
    set bx  [lindex $pr($i) 3]
    set by  [lindex $pr($i) 4]
    set print $pr($pr(printer)command)
    if {$s(gamma_support)==1} {
        set gamma "-gamma $s(gamma_red),$s(gamma_green),$s(gamma_blue)"
    } else {
        set gamma ""
    }
    switch $pr(paper) {
        A4     { set pwd 595. ; set pht 842. ; set offset 72 }
        Letter { set pwd 612. ; set pht 792. ; set offset 72 }
    }
    set rwd [pixelto pt [expr $bx-$tx]]
    set rht [pixelto pt [expr $by-$ty]]
    set zoom [expr $pr(zoom).0/100.0]
    set iwd [expr $rwd*$zoom]
    set iht [expr $rht*$zoom]
    if {$iwd > $pwd} {
        set iwd $pwd
        set zoom [expr $pwd/$rwd]
        set iht [expr $rht*$zoom]
    }
    if {$iht > $pht} {
        set iht $pht
        set zoom [expr $pht/$rht]
        set iwd [expr $rwd*$zoom]
    }
    set pr(zoom) [format "%0.0f" [expr $zoom*100]]
    set xspace [expr $pwd-$iwd]
    set yspace [expr $pht-$iht]
    switch $pr(position) {
        Center {
            set xoffset [expr ($pwd-$iwd)/2]
            set yoffset [expr ($pht-$iht)/2]
        }
        UpperLeft {
            if { $pr(xoffset) <= $xspace } {
                 set xoffset $pr(xoffset)
            } else {
                 set xoffset $xspace
                 set pr(xoffset) [format "%0.0f" $xspace]
            }
            if { $pr(yoffset) <= $yspace } {
                 set yoffset [expr $yspace-$pr(yoffset)]
            } else {
                 set yoffset 0
                 set pr(yoffset) [format "%0.0f" $yspace]
            }
        }
        UpperRight {
            if { $pr(xoffset) <= $xspace } {
                 set xoffset [expr $xspace-$pr(xoffset)]
            } else {
                 set xoffset 0
                 set pr(xoffset) [format "%0.0f" $xspace]
            }
            if { $pr(yoffset) <= $yspace } {
                 set yoffset [expr $yspace-$pr(yoffset)]
            } else {
                 set yoffset 0
                 set pr(yoffset) [format "%0.0f" $yspace]
            }
        }
        LowerLeft {
            if { $pr(xoffset) <= $xspace } {
                 set xoffset $pr(xoffset)
            } else {
                 set xoffset $xspace
                 set pr(xoffset) [format "%0.0f" $xspace]
            }
            if { $pr(yoffset) <= $yspace } {
                 set yoffset $pr(yoffset)
            } else {
                 set yoffset $yspace
                 set pr(yoffset) [format "%0.0f" $yspace]
            }
        }
        LowerRight {
            if { $pr(xoffset) <= $xspace } {
                 set xoffset [expr $xspace-$pr(xoffset)]
            } else {
                 set xoffset 0
                 set pr(xoffset) [format "%0.0f" $xspace]
            }
            if { $pr(yoffset) <= $yspace } {
                 set yoffset $pr(yoffset)
            } else {
                 set yoffset $yspace
                 set pr(yoffset) [format "%0.0f" $yspace]
            }
        }
    }
    set page [format "-page $pr(paper)+%0.0f+%0.0f" $xoffset $yoffset]
    set geometry [format "-geometry %0.0fx%0.0f" $iwd $iht]
    exec sh -c \
        "convert $gamma $page $geometry \
                 $c(tkscandir)/$tag.pnm $c(tkscandir)/$tag.ps ; \
         $print $c(tkscandir)/$tag.ps ; \
         rm -f $c(tkscandir)/$tag.ps" &
}

proc setPrinter { printer } { global pr w
    set pr(printer) $printer
    $w(printer) configure -text $printer
}

proc setPrintposition { position } { global pr s c w
    set pr(position) $position
    $w(printposition) configure -text $position
    if {$position == "Center"} {
        $w(xoffsetlabel) configure -fg gray
        $w(yoffsetlabel) configure -fg gray
    } else {
        $w(xoffsetlabel) configure -fg black
        $w(yoffsetlabel) configure -fg black
    }
}

